/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangBescheidWizardE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-bescheid-wizard.e2e.component';
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { VorgangE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from 'apps/alfa-e2e/src/model/vorgang';
import { uploadFile } from 'apps/alfa-e2e/src/support/file-upload';
import 'cypress-real-events/support';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections, wait } from '../../../support/cypress-helper';
import { contains, enterWith, exist, haveText, notExist } from '../../../support/cypress.util';
import { TEST_FILE_BESCHEID_VALID } from '../../../support/data.util';
import { getAdjustedDateEnglish, getAdjustedDateGerman } from '../../../support/tech.util';
import { initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { buildVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Bescheid Wizard', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const bewilligtText: string = 'Bewilligt am';
  const abgelehntText: string = 'Abgelehnt am';
  const stepCaption1: string = 'Antrag bescheiden';
  const stepCaption2: string = 'Dokumente hinzufügen';
  const stepCaption3: string = 'Bescheid versenden';

  const vorgangPage: VorgangPage = new VorgangPage();
  const bescheidWizard: VorgangBescheidWizardE2EComponent = vorgangPage.getBescheidWizard();

  const wizardVorgang: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'Wizard Vorgang'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
  };

  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent = vorgangPage.getFormularButtons();

  before(() => {
    initVorgaenge([wizardVorgang]);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('testing basic navigation elements', () => {
    it('should open Wizard after click on Bescheiden', () => {
      vorgangList.getListItem(wizardVorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      vorgangFormularButtons.getBescheidenButton().click();
      exist(bescheidWizard.getRoot());
    });

    it('should click on buttons and adjust text', () => {
      bescheidWizard.getAbgelehntButton().click();
      contains(bescheidWizard.getStatusText(), abgelehntText);

      bescheidWizard.getBewilligtButton().click();
      contains(bescheidWizard.getStatusText(), bewilligtText);

      enterWith(bescheidWizard.getDateInput(), getAdjustedDateGerman(-1));
      contains(bescheidWizard.getStatusText(), bewilligtText + ' ' + getAdjustedDateGerman(-1));
    });

    it('should show error message on invalid date format', () => {
      notExist(bescheidWizard.getDateError());

      enterWith(bescheidWizard.getDateInput(), getAdjustedDateEnglish(-1));
      bescheidWizard.getWeiterButton().click();
      waitForSpinnerToDisappear();
      exist(bescheidWizard.getDateError());

      enterWith(bescheidWizard.getDateInput(), getAdjustedDateGerman(-1));
    });

    it('should show next steps after click on Weiter', () => {
      bescheidWizard.getRoot().contains(stepCaption1);
      bescheidWizard.getRoot().should('not.contain', stepCaption2);
      bescheidWizard.getRoot().should('not.contain', stepCaption3);

      bescheidWizard.getWeiterButton().click();
      waitForSpinnerToDisappear();
      bescheidWizard.getRoot().contains(stepCaption1);
      bescheidWizard.getRoot().contains(stepCaption2);
      bescheidWizard.getRoot().should('not.contain', stepCaption3);

      uploadFile(bescheidWizard.getUploadBescheidButton(), TEST_FILE_BESCHEID_VALID);
      notExist(bescheidWizard.getBescheidUploadSpinner());
      bescheidWizard.getWeiterButton().click();
      waitForSpinnerToDisappear();
      bescheidWizard.getRoot().contains(stepCaption1);
      bescheidWizard.getRoot().contains(stepCaption2);
      bescheidWizard.getRoot().contains(stepCaption3);
    });

    it('should show previous steps after click on numbers', () => {
      bescheidWizard.getStepButton2().click();
      bescheidWizard.getRoot().contains(stepCaption1);
      bescheidWizard.getRoot().contains(stepCaption2);
      bescheidWizard.getRoot().should('not.contain', stepCaption3);

      bescheidWizard.getStepButton1().click();
      bescheidWizard.getRoot().contains(stepCaption1);
      bescheidWizard.getRoot().should('not.contain', stepCaption2);
      bescheidWizard.getRoot().should('not.contain', stepCaption3);
    });

    it('should close Wizard on Überspringen, discard data and set status to Abgeschlossen', () => {
      bescheidWizard.getAbgelehntButton().click();
      bescheidWizard.getWeiterButton().click();
      waitForSpinnerToDisappear();
      bescheidWizard.getStepButton1().click();
      bescheidWizard.getUeberspringenButton().click();
      exist(bescheidWizard.getUeberspringenDialog());

      bescheidWizard.getUeberspringenAbbrechen().click();
      notExist(bescheidWizard.getUeberspringenDialog());

      bescheidWizard.getUeberspringenButton().click();
      bescheidWizard.getUeberspringenAbschliessen().click();
      waitForSpinnerToDisappear();
      notExist(bescheidWizard.getRoot());

      wait(1000);
      haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN]);

      vorgangFormularButtons.getWiedereroeffnenButton().click();
      vorgangFormularButtons.getBescheidenButton().click();
      contains(bescheidWizard.getStatusText(), bewilligtText);
    });
  });
});
