/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { User } from '@admin-client/user-shared';
import { createEmptyStateResource, createStateResource, StateResource } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, getMockComponent, mock, Mock, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { DIALOG_COMPONENT } from '@alfa-client/ui';
import { CommonModule } from '@angular/common';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormGroup, ReactiveFormsModule } from '@angular/forms';
import { ButtonWithSpinnerComponent, SpinnerComponent } from '@ods/component';
import { cold } from 'jest-marbles';
import { createUser } from 'libs/admin/user-shared/test/user';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { getDataTestIdOf } from '../../../../../tech-shared/test/data-test';
import { UserFormDataComponent } from './user-form-data/user-form-data.component';
import { UserFormHeadlineComponent } from './user-form-headline/user-form-headline.component';
import { UserFormOrganisationsEinheitListComponent } from './user-form-organisations-einheit-list/user-form-organisations-einheit-list.component';
import { UserFormRolesComponent } from './user-form-roles/user-form-roles.component';
import { UserFormComponent } from './user-form.component';
import { UserFormService } from './user.formservice';

describe('UserFormComponent', () => {
  let component: UserFormComponent;
  let fixture: ComponentFixture<UserFormComponent>;

  let formService: Mock<UserFormService>;

  const userContent: string = getDataTestIdOf('user-content');
  const deleteButtonContainer: string = getDataTestIdOf('delete-button-container');

  beforeEach(async () => {
    formService = <any>{
      ...mock(UserFormService),
      get: jest.fn(),
      isInvalid: jest.fn(),
      form: new FormGroup({}),
      isPatch: jest.fn(),
    };

    await TestBed.configureTestingModule({
      imports: [CommonModule, ReactiveFormsModule, SpinnerComponent],
      declarations: [
        UserFormComponent,
        MockComponent(ButtonWithSpinnerComponent),
        MockComponent(UserFormDataComponent),
        MockComponent(UserFormOrganisationsEinheitListComponent),
        MockComponent(UserFormRolesComponent),
        MockComponent(UserFormHeadlineComponent),
      ],
      providers: [{ provide: DIALOG_COMPONENT, useValue: {} }],
    })
      .overrideComponent(UserFormComponent, {
        set: {
          providers: [
            {
              provide: UserFormService,
              useValue: formService,
            },
          ],
        },
      })
      .compileComponents();

    fixture = TestBed.createComponent(UserFormComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('on init', () => {
      const userStateResource: StateResource<User> = createStateResource(createUser());

      beforeEach(() => {
        formService.get.mockReturnValue(of(userStateResource));
      });

      it('should call formservice to get user', () => {
        component.ngOnInit();

        expect(formService.get).toHaveBeenCalled();
      });

      it('should set user state resource', () => {
        component.userStateResource$ = of(createEmptyStateResource<User>());

        component.ngOnInit();

        expect(component.userStateResource$).toBeObservable(cold('(a|)', { a: userStateResource }));
      });
    });
  });

  describe('template', () => {
    describe('admin headline', () => {
      it('should exist with input', () => {
        formService.isPatch.mockReturnValue(true);

        fixture.detectChanges();

        const headlineComponent: UserFormHeadlineComponent = getMockComponent(fixture, UserFormHeadlineComponent);
        expect(headlineComponent.isPatch).toBe(true);
      });
    });

    describe('admin data', () => {
      it('should exist with input', () => {
        const formDataComponent: UserFormDataComponent = getMockComponent(fixture, UserFormDataComponent);

        expect(formDataComponent.formGroupParent).toBe(component.formService.form);
      });
    });

    describe('admin roles', () => {
      it('should exist with input', () => {
        const rolesComponent: UserFormRolesComponent = getMockComponent(fixture, UserFormRolesComponent);

        expect(rolesComponent).toBeTruthy();
        expect(rolesComponent.formGroupParent).toBe(component.formService.form);
      });
    });

    describe('admin organisations einheiten', () => {
      it('should exist with input', () => {
        const organisationsEinheitListComponent: UserFormOrganisationsEinheitListComponent = getMockComponent(
          fixture,
          UserFormOrganisationsEinheitListComponent,
        );

        expect(organisationsEinheitListComponent.formGroupParent).toBe(component.formService.form);
        expect(organisationsEinheitListComponent.formGroupOrganisationsEinheiten).toBe(
          component.formService.getOrganisationsEinheitenGroup(),
        );
      });
    });

    it('should hide content on loading user', () => {
      component.userStateResource$ = of(createEmptyStateResource<User>(true));

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, userContent);
    });
  });

  describe('admin delete button container', () => {
    it('should exist', () => {
      formService.isPatch.mockReturnValue(true);

      fixture.detectChanges();

      existsAsHtmlElement(fixture, deleteButtonContainer);
    });

    it('should not exist', () => {
      formService.isPatch.mockReturnValue(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, deleteButtonContainer);
    });
  });
});
