import { BescheidLinkRel, BescheidWizardDialogResult } from '@alfa-client/bescheid-shared';
import { BinaryFileListContainerComponent } from '@alfa-client/binary-file';
import {
  createEmptyStateResource,
  createErrorStateResource,
  createLoadingStateResource,
  HasLinkPipe,
} from '@alfa-client/tech-shared';
import { existsAsHtmlElement, mock, Mock, notExistsAsHtmlElement, useFromMock } from '@alfa-client/test-utils';
import { DialogRef } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder, ReactiveFormsModule } from '@angular/forms';
import { expect } from '@jest/globals';
import { ButtonWithSpinnerComponent, TextareaEditorComponent, TextEditorComponent } from '@ods/component';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { BescheidService2 } from '../../../../../../../../bescheid-shared/src/lib/bescheid2.service';
import { createBescheidResource, createWizard } from '../../../../../../../../bescheid-shared/src/test/bescheid';
import { createSuccessfullyDoneCommandStateResource } from '../../../../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../../../../tech-shared/test/data-test';
import { createProblemDetail } from '../../../../../../../../tech-shared/test/error';
import { singleColdCompleted } from '../../../../../../../../tech-shared/test/marbles';
import { BescheidFormService } from '../../../../bescheid.formservice';
import { BescheidWizardDocumentFileContainerComponent } from '../../../document-file-container/bescheid-wizard-document-file-container.component';
import { BescheidWizardBescheidVersendenSendenComponent } from './bescheid-wizard-bescheid-versenden-senden.component';

describe('BescheidWizardBescheidVersendenSendenComponent', () => {
  let component: BescheidWizardBescheidVersendenSendenComponent;
  let fixture: ComponentFixture<BescheidWizardBescheidVersendenSendenComponent>;

  const attachmentsTestId: string = getDataTestIdOf('bescheid-attachments');
  const empfaengerTestId: string = getDataTestIdOf('bescheid-nachricht-empfaenger');

  let bescheidService: Mock<BescheidService2>;
  let formService: BescheidFormService;
  let dialogRef: Mock<DialogRef<BescheidWizardDialogResult>>;

  beforeEach(() => {
    bescheidService = mock(BescheidService2);
    formService = new BescheidFormService(new FormBuilder(), useFromMock(bescheidService));
    formService.submit = jest.fn();
    dialogRef = mock(DialogRef<BescheidWizardDialogResult>);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardBescheidVersendenSendenComponent,
        MockComponent(ButtonWithSpinnerComponent),
        MockComponent(TextareaEditorComponent),
        MockComponent(TextEditorComponent),
        MockComponent(BinaryFileListContainerComponent),
        MockComponent(BescheidWizardDocumentFileContainerComponent),
        HasLinkPipe,
        ReactiveFormsModule,
      ],
      providers: [
        { provide: BescheidFormService, useValue: formService },
        { provide: DialogRef<BescheidWizardDialogResult>, useValue: dialogRef },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardBescheidVersendenSendenComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  describe('component', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    it('should set initial values', () => {
      expect(component.submitStateResource$).toBeObservable(singleColdCompleted(createEmptyStateResource()));
    });

    describe('submit', () => {
      beforeEach(() => {
        formService.submit = jest.fn().mockReturnValue(EMPTY);
        component._resetFocus = jest.fn();
      });

      it('should reset focus', () => {
        component.submit();
        component.submitStateResource$.subscribe();

        expect(component._resetFocus).toHaveBeenCalled();
      });

      it('should call form service', () => {
        component.submit();
        component.submitStateResource$.subscribe();

        expect(formService.submit).toHaveBeenCalled();
      });

      it('should close dialog', () => {
        formService.submit = jest.fn().mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));

        component.submit();
        component.submitStateResource$.subscribe();

        expect(dialogRef.close).toHaveBeenCalledWith({ reloadVorgang: true } as BescheidWizardDialogResult);
      });

      it('should NOT close dialog on pending send', () => {
        formService.submit = jest.fn().mockReturnValue(of(createLoadingStateResource()));

        component.submit();
        component.submitStateResource$.subscribe();

        expect(dialogRef.close).not.toHaveBeenCalled();
      });

      it('should NOT close dialog on failed send', () => {
        formService.submit = jest.fn().mockReturnValue(of(createErrorStateResource(createProblemDetail())));

        component.submit();
        component.submitStateResource$.subscribe();

        expect(dialogRef.close).not.toHaveBeenCalled();
      });

      it('should focus to first invalid control', () => {
        component._focusToFirstInvalidControl = jest.fn();
        formService.submit = jest.fn().mockReturnValue(of(createErrorStateResource(createProblemDetail())));

        component.submit();
        component.submitStateResource$.subscribe();

        expect(component._focusToFirstInvalidControl).toHaveBeenCalled();
      });

      it('should NOT focus to first invalid control on pending submit', () => {
        component._focusToFirstInvalidControl = jest.fn();
        formService.submit = jest.fn().mockReturnValue(of(createLoadingStateResource()));

        component.submit();
        component.submitStateResource$.subscribe();

        expect(component._focusToFirstInvalidControl).not.toHaveBeenCalled();
      });

      it('should NOT focus to first invalid control on successful submit', () => {
        component._focusToFirstInvalidControl = jest.fn();
        formService.submit = jest.fn().mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));

        component.submit();
        component.submitStateResource$.subscribe();

        expect(component._focusToFirstInvalidControl).not.toHaveBeenCalled();
      });
    });

    describe('_focusToFirstInvalidControl', () => {
      beforeEach(() => {
        component._resetFocus = jest.fn();

        formService.isBetreffInvalid = jest.fn();
        formService.isNachrichtInvalid = jest.fn();
      });

      it('should check betreff', () => {
        component._focusToFirstInvalidControl();

        expect(formService.isBetreffInvalid).toHaveBeenCalled();
      });

      it('should check nachricht', () => {
        formService.isBetreffInvalid = jest.fn().mockReturnValue(false);

        component._focusToFirstInvalidControl();

        expect(formService.isNachrichtInvalid).toHaveBeenCalled();
      });

      it('should NOT check nachricht', () => {
        formService.isBetreffInvalid = jest.fn().mockReturnValue(true);

        component._focusToFirstInvalidControl();

        expect(formService.isNachrichtInvalid).not.toHaveBeenCalled();
      });

      it('should focus betreff', () => {
        formService.isBetreffInvalid = jest.fn().mockReturnValue(true);

        component._focusToFirstInvalidControl();

        expect(component.focusBetreff).toBe(true);
      });

      it('should NOT focus betreff', () => {
        formService.isBetreffInvalid = jest.fn().mockReturnValue(false);

        component._focusToFirstInvalidControl();

        expect(component.focusBetreff).toBe(false);
      });

      it('should focus nachricht', () => {
        formService.isBetreffInvalid = jest.fn().mockReturnValue(false);
        formService.isNachrichtInvalid = jest.fn().mockReturnValue(true);

        component._focusToFirstInvalidControl();

        expect(component.focusNachricht).toBe(true);
      });

      it('should NOT focus nachricht', () => {
        formService.isBetreffInvalid = jest.fn().mockReturnValue(true);
        formService.isNachrichtInvalid = jest.fn().mockReturnValue(false);

        component._focusToFirstInvalidControl();

        expect(component.focusNachricht).toBe(false);
      });
    });
  });

  describe('template', () => {
    describe('empfaenger', () => {
      it('should exists', () => {
        component.wizard = createWizard();

        fixture.detectChanges();

        existsAsHtmlElement(fixture, empfaengerTestId);
      });

      it('should NOT exits', () => {
        component.wizard = { ...createWizard(), empfaenger: null };

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, empfaengerTestId);
      });
    });

    describe('attachment list', () => {
      it('should exists', () => {
        component.bescheidResource = createBescheidResource([BescheidLinkRel.ATTACHMENTS]);

        fixture.detectChanges();

        existsAsHtmlElement(fixture, attachmentsTestId);
      });

      it('should NOT exists', () => {
        component.bescheidResource = createBescheidResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, attachmentsTestId);
      });
    });
  });
});
