/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidDocument, BescheidResource, BescheidWizardStep } from '@alfa-client/bescheid-shared';
import { getElementComponentFromFixtureByCss, Mock, mock, triggerEvent } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { expect } from '@jest/globals';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BescheidService2 } from '../../../../../../bescheid-shared/src/lib/bescheid2.service';
import { createBescheidDocument, createBescheidResource } from '../../../../../../bescheid-shared/src/test/bescheid';
import { getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import { singleColdCompleted } from '../../../../../../tech-shared/test/marbles';
import { BescheidWizardAntragBescheidenSummaryComponent } from '../antrag-bescheiden/summary/bescheid-wizard-antrag-bescheiden-summary.component';
import { StepContentLayoutComponent } from '../step-content-layout/step-content-layout.component';
import { BescheidWizardStepTitleComponent } from '../step-title/bescheid-wizard-step-title.component';
import { BescheidWizardStepperComponent } from '../stepper/bescheid-wizard-stepper.component';
import { BescheidWizardSummaryComponent } from '../summary/bescheid-wizard-summary.component';
import { BescheidWizardDokumenteHochladenContainerComponent } from './bescheid-wizard-dokumente-hochladen-container.component';
import { BescheidWizardDokumenteHochladenFormComponent } from './form/bescheid-wizard-dokumente-hochladen-form.component';
import { BescheidWizardDokumenteHochladenSummaryComponent } from './summary/bescheid-wizard-dokumente-hochladen-summary.component';

describe('BescheidWizardDokumenteHochladenComponent', () => {
  let component: BescheidWizardDokumenteHochladenContainerComponent;
  let fixture: ComponentFixture<BescheidWizardDokumenteHochladenContainerComponent>;

  const stepContentLayout: string = getDataTestIdOf('step-content-layout');
  const antragBescheidenStepTitleTestId: string = getDataTestIdOf('antrag-bescheiden-step-title');
  const dokumenteHochladenFormTestId: string = getDataTestIdOf('bescheid-wizard-dokumente-hochladen-form');
  const dokumenteHochladenSummaryTestId: string = getDataTestIdOf('bescheid-wizard-dokumente-hochladen-summary');

  const bescheidResource: BescheidResource = createBescheidResource();
  const bescheidDocument: BescheidDocument = createBescheidDocument();

  let bescheidService: Mock<BescheidService2>;

  beforeEach(() => {
    bescheidService = mock(BescheidService2);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardDokumenteHochladenContainerComponent,
        MockComponent(StepContentLayoutComponent),
        MockComponent(BescheidWizardSummaryComponent),
        MockComponent(BescheidWizardAntragBescheidenSummaryComponent),
        MockComponent(BescheidWizardDokumenteHochladenSummaryComponent),
        MockComponent(BescheidWizardStepTitleComponent),
        MockComponent(BescheidWizardDokumenteHochladenFormComponent),
        MockComponent(BescheidWizardStepperComponent),
      ],
      providers: [{ provide: BescheidService2, useValue: bescheidService }],
    }).compileComponents();

    createComponent();
  });

  function createComponent() {
    fixture = TestBed.createComponent(BescheidWizardDokumenteHochladenContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }

  describe('component', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    it('should set initial values', () => {
      bescheidService.getBescheidDocument.mockReturnValue(of(bescheidDocument));

      createComponent();

      expect(component.bescheidDocument$).toBeObservable(singleColdCompleted(bescheidDocument));
    });

    describe('ngOnInit', () => {
      it('should load files', () => {
        component.ngOnInit();

        expect(bescheidService.loadFiles).toHaveBeenCalled();
      });
    });

    describe('set bescheid resource', () => {
      it('should set value', () => {
        component.bescheidResource = bescheidResource;

        expect(component.bescheidResource).toEqual(bescheidResource);
      });
    });

    describe('onWeiterClick', () => {
      it('should finish adding bescheid document', () => {
        component.onWeiterClick();

        expect(bescheidService.finishAddingBescheidDocument).toHaveBeenCalled();
      });

      it('should change active step', () => {
        component.onWeiterClick();

        expect(bescheidService.setActiveStep).toHaveBeenCalledWith(BescheidWizardStep.BescheidVersenden);
      });
    });

    describe('onBescheidDocumentMissing', () => {
      it('should set value', () => {
        component.isBescheidDocumentMissing = false;

        component.onBescheidDocumentMissing(true);

        expect(component.isBescheidDocumentMissing).toEqual(true);
      });
    });

    describe('onStepChange', () => {
      it('should change step', () => {
        component.onStepChange(BescheidWizardStep.AntragBescheiden);

        expect(bescheidService.setActiveStep).toHaveBeenCalledWith(BescheidWizardStep.AntragBescheiden);
      });
    });
  });

  describe('template', () => {
    describe('step content layout', () => {
      describe('output', () => {
        it('should call handler', () => {
          component.onStepChange = jest.fn();

          triggerEvent({
            fixture,
            name: 'stepChange',
            elementSelector: stepContentLayout,
            data: BescheidWizardStep.AntragBescheiden,
          });

          expect(component.onStepChange).toHaveBeenCalledWith(BescheidWizardStep.AntragBescheiden);
        });
      });
    });

    describe('antrag bescheiden step title', () => {
      it('should have been called with inputs', () => {
        const stepTitle: BescheidWizardStepTitleComponent = getElementComponentFromFixtureByCss<BescheidWizardStepTitleComponent>(
          fixture,
          antragBescheidenStepTitleTestId,
        );

        expect(stepTitle.inactiveStep).toBeTruthy();
      });
    });

    describe('hochladen form', () => {
      it('should have been called with inputs', () => {
        bescheidService.getBescheidDocument.mockReturnValue(of(bescheidDocument));
        createComponent();
        component.bescheidResource = bescheidResource;

        fixture.detectChanges();
        const hochladenForm: BescheidWizardDokumenteHochladenFormComponent =
          getElementComponentFromFixtureByCss<BescheidWizardDokumenteHochladenFormComponent>(
            fixture,
            dokumenteHochladenFormTestId,
          );

        expect(hochladenForm.bescheidResource).toEqual(bescheidResource);
        expect(hochladenForm.bescheidDocument).toEqual(bescheidDocument);
      });

      describe('output', () => {
        describe('weiterClickEmitter', () => {
          it('should call handler', () => {
            component.onWeiterClick = jest.fn();

            triggerEvent({ fixture, elementSelector: dokumenteHochladenFormTestId, name: 'weiterClickEmitter' });

            expect(component.onWeiterClick).toHaveBeenCalled();
          });
        });

        describe('bescheidDocumentMissing', () => {
          it('should call handler', () => {
            component.onBescheidDocumentMissing = jest.fn();

            triggerEvent({ fixture, elementSelector: dokumenteHochladenFormTestId, name: 'bescheidDocumentMissing' });

            expect(component.onBescheidDocumentMissing).toHaveBeenCalled();
          });
        });
      });
    });

    describe('hochladen summary', () => {
      it('should have been called with inputs', () => {
        component.isBescheidDocumentMissing = true;

        fixture.detectChanges();
        const hochladenSummary: BescheidWizardDokumenteHochladenSummaryComponent =
          getElementComponentFromFixtureByCss<BescheidWizardDokumenteHochladenSummaryComponent>(
            fixture,
            dokumenteHochladenSummaryTestId,
          );

        expect(hochladenSummary).toBeTruthy();
      });
    });
  });
});
